@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2012-2015 Calculate Ltd. http://www.calculate-linux.org
+# Copyright 2012-2016 Mir Calculate. http://www.calculate-linux.org
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
 #  you may not use this file except in compliance with the License.
@@ -26,7 +26,7 @@ setLocalTranslate('cl_core3', sys.modules[__name__])
 
 
 def passphrase_callback(v):
-    return None
+    return ""
 
 
 def generateRSAKey():
@@ -93,7 +93,7 @@ def makeRequest(pubkey, pkey, serv_host, auto=False):
         name.C = lang  # Двухсимвольный код страны (Country);
     req.set_subject_name(name)
 
-    ext1 = X509.new_extension('Comment', 'Auto Generated')
+    ext1 = X509.new_extension('nsComment', 'Auto Generated')
     extstack = X509.X509_Extension_Stack()
     extstack.push(ext1)
     req.add_extensions(extstack)
