@@ -17,13 +17,25 @@
 import sys
 from M2Crypto import RSA, X509, EVP, m2
 from calculate.lib.cl_lang import setLocalTranslate
+from binascii import hexlify
+import hashlib
+
+'''
+Copyright (c) 2016 Jesse Peterson
+Licensed under the MIT license. See the included LICENSE.txt file for details.
+'''
+ 
+from M2Crypto import m2
+from M2Crypto.X509 import X509_Extension
+from ctypes import *
+ 
 
 _ = lambda x: x
 setLocalTranslate('cl_core3', sys.modules[__name__])
 
 
 def passphrase_callback(v):
-    return None
+    return ""
 
 
 def generateRSAKey():
@@ -95,60 +107,68 @@ def makeRequest(pubkey, pkey, serv_host, port):
         name.C = raw_input(_('Country (two letters only!): '))
 
     req.set_subject_name(name)
-    # ext1 = X509.new_extension('nsComment', 'Hello there')
-    ext1 = X509.new_extension('Comment', 'Auto Generated')
+    ext1 = X509.new_extension('nsComment', 'Auto Generated')
     extstack = X509.X509_Extension_Stack()
     extstack.push(ext1)
     req.add_extensions(extstack)
     req.sign(pkey, 'md5')
     return req
 
+class CreateCertError(Exception):
+    pass
 
-def makeCert(req, caPkey, issuer, client_cert=False, group=None):
-    pkey = req.get_pubkey()
-    # woop = makePKey(generateRSAKey())
-    # if not req.verify(woop.pkey):
-    if not req.verify(pkey):
-        # XXX What error object should I use?
-        raise ValueError('Error verifying request')
-    sub = req.get_subject()
-    # If this were a real certificate request, you would display
-    # all the relevant data from the request and ask a human operator
-    # if you were sure. Now we just create the certificate blindly based
-    # on the request.    
-    cert = X509.X509()
-    # We know we are making CA cert now...
-    # Serial defaults to 0.
-    cert.set_serial_number(1)
-    cert.set_version(2)
-    cert.set_subject(sub)
-    cert.set_issuer(issuer)
-    cert.set_pubkey(pkey)
-    notBefore = m2.x509_get_not_before(cert.x509)
-    notAfter = m2.x509_get_not_after(cert.x509)
-    m2.x509_gmtime_adj(notBefore, -1 * 60 * 60 * 24)
-    days = 1095  # 3 year
-    m2.x509_gmtime_adj(notAfter, 60 * 60 * 24 * days)
+def create_selfsigned_ca(dn_data, keyfile, certfile):
+    from calculate.lib.utils.gpg import Pipe
+    from calculate.lib.utils.files import process
+    from string import Template
+    ca_conf = Template("""
+[ req ]
+default_bits = 2048
+distinguished_name = req_distinguished_name
+prompt = no
+x509_extensions = v3_ca
 
-    if client_cert:
-        cert.add_ext(X509.new_extension('basicConstraints', 'CA:FALSE'))
-        cert.add_ext(X509.new_extension('nsCertType', 'client'))
-        cert.add_ext(X509.new_extension('keyUsage',
-                                        'digitalSignature, keyEncipherment'))
-        cert.add_ext(X509.new_extension('extendedKeyUsage', 'clientAuth'))
-        if group:
-            cert.add_ext(X509.new_extension('nsComment', 'group:%s' % group))
-    else:
-        ext = X509.new_extension('basicConstraints', 'CA:TRUE')
-        ext.set_critical(1)
-        cert.add_ext(ext)
+[ req_distinguished_name ]
+CN = $CN
+ST = $ST
+L = $L
+O = $O
+OU = $OU
 
-        # Create the subject key identifier
-        # modulus = cert.get_pubkey().get_modulus()
-        # sha_hash = hashlib.sha1(modulus).digest()
-        # sub_key_id = ":".join(["%02X"%ord(byte) for byte in sha_hash])
-        cert.add_ext(X509.new_extension('subjectKeyIdentifier', 'hash'))
+[ v3_ca ]
+subjectKeyIdentifier=hash
+authorityKeyIdentifier=keyid:always,issuer
+basicConstraints = critical,CA:true
+""")
+    with Pipe() as cfgpipe:
+        p = process("openssl", "req", "-x509", "-new", "-key", keyfile,
+                    "-config", cfgpipe.get_filename(), "-out", certfile,
+                    "-days", "7300")
+        cfgpipe.write(ca_conf.safe_substitute(dn_data))
+        cfgpipe.closein()
+        if p.failed():
+            raise CreateCertError(p.readerr())
 
-        cert.add_ext(X509.new_extension('authorityKeyIdentifier', 'keyid'))
-    cert.sign(caPkey, 'sha1')
-    return cert
+def sign_client_certifacation_request(ca_keyfile, ca_certfile, requestfile, out_cert, group):
+    from calculate.lib.utils.gpg import Pipe
+    from calculate.lib.utils.files import process
+    from string import Template
+    client_conf = Template("""
+[ client ]
+nsCertType = client
+keyUsage = digitalSignature, keyEncipherment
+extendedKeyUsage = clientAuth
+nsComment = group:$group
+""")
+    with Pipe() as cfgpipe:
+        p = process("openssl", "x509", "-req", "-in", requestfile,
+                    "-CA", ca_certfile,
+                    "-CAkey", ca_keyfile, "-CAcreateserial",
+                    "-extfile", cfgpipe.get_filename(),
+                    "-extensions", "client",
+                    "-out", out_cert,
+                    "-days", "7300")
+        cfgpipe.write(client_conf.safe_substitute({"group":group}))
+        cfgpipe.closein()
+        if p.failed():
+            raise CreateCertError(p.readerr())
